@@ -16,6 +16,7 @@ from pynfra.modules.db import DB
 from pynfra.modules.ssh_keys import generate_key, remove_hostkeys
 from pynfra.modules.node import Node, NodeAuthException, NodeTimeoutException
 from pynfra.modules.tui import TUI
+from pynfra.modules.classes import RecipeBase
 
 
 class Pynfra:
@@ -25,7 +26,7 @@ class Pynfra:
 
         # parse arguments
         self.__parser = None
-        self.__args = self.__parse_args()
+        self.__args, self.__unknownargs = self.__parse_args()
 
         # config paths
         self.__CONF_DIR = self.__args.confdir
@@ -37,6 +38,7 @@ class Pynfra:
         # vars
         self.__db = DB(self.__CONFIGNAME, self.__HOSTKEYS)
         self.__first_run = self.__db.is_first_run()
+        self.__recipes = {}
 
     def __node_exec(self, cmd, nodes):
         def run(node, cmd):
@@ -67,6 +69,12 @@ class Pynfra:
             return f(self, *args, **kwargs)
         return magic
 
+    def load_recipe(self, name, recipe):
+        if not issubclass(recipe, RecipeBase):
+            print("[-] Recipe {} has to be of type {}".format(recipe, RecipeBase))
+            exit(1)
+        self.__recipes[name] = recipe
+
     @need_init
     def add_server(self, hostname, address, password=None, keyfile=None, overwrite=False):
         print("[+] Adding server {}, password: {}".format(address, password))
@@ -405,9 +413,17 @@ class Pynfra:
         node.copyfile(direction, src, dst)
 
     @need_init
-    def recipe(self, recipename):
-        print("hi")
-        pass
+    def recipe(self, recipename, nodename, *args):
+        if recipename not in self.__recipes:
+            print("Recipe \"{}\" not found, available recipes:".format(recipename))
+            for name, recipe in self.__recipes.items():
+                print("\t{}".format(name))
+                exit(1)
+
+        node = self.__get_server(nodename)
+
+        recipe = self.__recipes[recipename](node)
+        recipe.main(args)
 
     def __parse_args(self):
         # if --confdir is set use it
@@ -491,10 +507,11 @@ class Pynfra:
 
         parser_recipe = subparsers.add_parser('recipe', help='specify recipe to load')
         parser_recipe.add_argument('recipe', type=str)
+        parser_recipe.add_argument('hostname', type=str)
         parser_recipe.set_defaults(func='recipe')
 
-        args = self.__parser.parse_args()
-        return args
+        args, unkownargs = self.__parser.parse_known_args()
+        return args, unkownargs
 
     def main(self):
         func = self.__args.func
@@ -527,7 +544,7 @@ class Pynfra:
         elif func == "run-all":
             self.exec(None, self.__args.cmd)
         elif func == "recipe":
-            self.recipe(self.__args.recipe)
+            self.recipe(self.__args.recipe, self.__args.hostname, self.__unknownargs)
         else:
             self.__parser.print_help()
 
